/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef CORRECTIONDAHEADERFILE
#define CORRECTIONDAHEADERFILE

#include "../../core/vector2.h"

class TCorrectionData
{
public:
// CREATORS
	TCorrectionData(void);
	TCorrectionData(const TVector2 &p1,const TVector2 &p2,const TVector2 &p3);
	TCorrectionData(const TCorrectionData &rhs);
	TCorrectionData& operator=(const TCorrectionData &rhs);
	~TCorrectionData(void);
	
// ACCESSORS
	TVector2 Point1(void) const {return d_data_pnts[0];}
	TVector2 Point2(void) const {return d_data_pnts[1];}
	TVector2 Point3(void) const {return d_data_pnts[2];}
	TVector2 Value(const double &pos) const;
	bool Valid_Range(void) const;
	
// MANIPULATORS
	void Set_Points(const TVector2 &p1,const TVector2 &p2,const TVector2 &p3);
	void Set_Point1(const TVector2 &pnt);
	void Set_Point2(const TVector2 &pnt);
	void Set_Point3(const TVector2 &pnt);
	
private:
	
	TVector2							d_data_pnts[3];
	double								d_data_coeff[3];	// a,b,c in that order.  ax^2 + bx + c
	
	void Update_Coefficients(void);
	
// NOT IMPLEMENTED
};


#endif
