/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////


#include <QApplication>
#include <QFrame>
#include <QGridLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QPushButton>
#include <QSpacerItem>
#include <QDateEdit>

#include "correctiondialog.h"

TCorrectionDialog::TCorrectionDialog(
	const QWidget						*parent,
	const Qt::WindowFlags				flags)
:QDialog(const_cast<QWidget*>(parent),flags)
{
	QFrame								*separator_line_lower;
	QFrame								*separator_line_upper;
	QGridLayout							*dialog_layout;
	QHBoxLayout							*button_hlayout;
	QLabel								*calibration_due_label;
	QLabel								*description_label;
	QLabel								*meter_reading_label;
	QLabel								*reference_temperature_label;
	QLabel								*test_point_1_label;
	QLabel								*test_point_2_label;
	QLabel								*test_point_3_label;
	QPushButton							*cancel_button;
	QSpacerItem							*button_hspacer;
	QSpacerItem							*dialog_vspacer;


	this->resize(601,301);

	dialog_layout = new QGridLayout(this);
	
	description_label = new QLabel(this);
	dialog_layout->addWidget(description_label,0,0,1,1);
	
	d_description_edit = new QLineEdit(this);
	dialog_layout->addWidget(d_description_edit,0,1,1,2);
	
	calibration_due_label = new QLabel(this);
	dialog_layout->addWidget(calibration_due_label,1,0,1,1);
	
	d_calibration_due_edit = new QDateEdit(this);
	dialog_layout->addWidget(d_calibration_due_edit,1,1,1,2);

	separator_line_upper = new QFrame(this);
	separator_line_upper->setFrameShape(QFrame::HLine);
	separator_line_upper->setFrameShadow(QFrame::Sunken);
	dialog_layout->addWidget(separator_line_upper,2,0,1,3);
	
	reference_temperature_label = new QLabel(this);
	reference_temperature_label->setAlignment(Qt::AlignCenter);
	dialog_layout->addWidget(reference_temperature_label,3,1,1,1);

	meter_reading_label = new QLabel(this);
	meter_reading_label->setAlignment(Qt::AlignCenter);
	dialog_layout->addWidget(meter_reading_label,3,2,1,1);

	test_point_1_label = new QLabel(this);
	dialog_layout->addWidget(test_point_1_label,4,0,1,1);
	
	d_reference_temperature_1_edit = new QLineEdit(this);
	d_reference_temperature_1_edit->setAlignment(Qt::AlignCenter);
	dialog_layout->addWidget(d_reference_temperature_1_edit,4,1,1,1);

	d_meter_temperature_1_edit = new QLineEdit(this);
	d_meter_temperature_1_edit->setAlignment(Qt::AlignCenter);
	dialog_layout->addWidget(d_meter_temperature_1_edit,4,2,1,1);

	test_point_2_label = new QLabel(this);
	dialog_layout->addWidget(test_point_2_label,5,0,1,1);

	d_reference_temperature_2_edit = new QLineEdit(this);
	d_reference_temperature_2_edit->setAlignment(Qt::AlignCenter);
	dialog_layout->addWidget(d_reference_temperature_2_edit,5,1,1,1);

	d_meter_temperature_2_edit = new QLineEdit(this);
	d_meter_temperature_2_edit->setAlignment(Qt::AlignCenter);
	dialog_layout->addWidget(d_meter_temperature_2_edit,5,2,1,1);

	test_point_3_label = new QLabel(this);
	dialog_layout->addWidget(test_point_3_label,6,0,1,1);

	d_reference_temperature_3_edit = new QLineEdit(this);
	d_reference_temperature_3_edit->setAlignment(Qt::AlignCenter);
	dialog_layout->addWidget(d_reference_temperature_3_edit,6,1,1,1);

	d_meter_temperature_3_edit = new QLineEdit(this);
	d_meter_temperature_3_edit->setAlignment(Qt::AlignCenter);
	dialog_layout->addWidget(d_meter_temperature_3_edit,6,2,1,1);

	separator_line_lower = new QFrame(this);
	separator_line_lower->setFrameShape(QFrame::HLine);
	separator_line_lower->setFrameShadow(QFrame::Sunken);
	dialog_layout->addWidget(separator_line_lower,7,0,1,3);
	
	dialog_vspacer = new QSpacerItem(0,0,QSizePolicy::Minimum,QSizePolicy::Expanding);
	dialog_layout->addItem(dialog_vspacer,8,0,1,3);

	button_hlayout = new QHBoxLayout();

	button_hspacer = new QSpacerItem(0,0,QSizePolicy::Expanding,QSizePolicy::Minimum);
	button_hlayout->addItem(button_hspacer);

	cancel_button = new QPushButton(this);
	cancel_button->setAutoDefault(false);
	button_hlayout->addWidget(cancel_button);

	d_accept_button = new QPushButton(this);
	d_accept_button->setAutoDefault(false);
	button_hlayout->addWidget(d_accept_button);
	dialog_layout->addLayout(button_hlayout,9,0,1,3);

	QWidget::setTabOrder(d_description_edit,d_calibration_due_edit);
	QWidget::setTabOrder(d_calibration_due_edit,d_reference_temperature_1_edit);
	QWidget::setTabOrder(d_reference_temperature_1_edit,d_meter_temperature_1_edit);
	QWidget::setTabOrder(d_meter_temperature_1_edit,d_reference_temperature_2_edit);
	QWidget::setTabOrder(d_reference_temperature_2_edit,d_meter_temperature_2_edit);
	QWidget::setTabOrder(d_meter_temperature_2_edit,d_reference_temperature_3_edit);
	QWidget::setTabOrder(d_reference_temperature_3_edit,d_meter_temperature_3_edit);
	QWidget::setTabOrder(d_meter_temperature_3_edit,cancel_button);
	QWidget::setTabOrder(cancel_button,d_accept_button);

	this->setWindowTitle(QStringLiteral("Correction Editor"));
	
	description_label->setText(QStringLiteral("Thermometer Description:"));
	calibration_due_label->setText(QStringLiteral("Calibration Due Date:"));
	d_calibration_due_edit->setDisplayFormat(QStringLiteral("MMM dd yyyy"));
	reference_temperature_label->setText(QStringLiteral("Reference Temperature ˚C"));
	meter_reading_label->setText(QStringLiteral("Thermometer Reading ˚C"));
	test_point_1_label->setText(QStringLiteral("Test Point 1:"));
	test_point_2_label->setText(QStringLiteral("Test Point 2:"));
	test_point_3_label->setText(QStringLiteral("Test Point 3:"));
	cancel_button->setText(QStringLiteral("Cancel"));
	d_accept_button->setText(QStringLiteral("Accept"));
	
	connect(cancel_button,&QPushButton::clicked,this,&TCorrectionDialog::close);
	connect(d_accept_button,&QPushButton::clicked,this,&TCorrectionDialog::accept);

	connect(d_description_edit,&QLineEdit::editingFinished,this,&TCorrectionDialog::Description_Changed);
	connect(d_calibration_due_edit,&QDateEdit::dateChanged,this,&TCorrectionDialog::Calibration_Date_Changed);
	connect(d_reference_temperature_1_edit,&QLineEdit::editingFinished,this,&TCorrectionDialog::Temperature_Changed);
	connect(d_reference_temperature_2_edit,&QLineEdit::editingFinished,this,&TCorrectionDialog::Temperature_Changed);
	connect(d_reference_temperature_3_edit,&QLineEdit::editingFinished,this,&TCorrectionDialog::Temperature_Changed);
	connect(d_meter_temperature_1_edit,&QLineEdit::editingFinished,this,&TCorrectionDialog::Temperature_Changed);
	connect(d_meter_temperature_2_edit,&QLineEdit::editingFinished,this,&TCorrectionDialog::Temperature_Changed);
	connect(d_meter_temperature_3_edit,&QLineEdit::editingFinished,this,&TCorrectionDialog::Temperature_Changed);
}

TCorrectionDialog::~TCorrectionDialog(void)
{
}

void TCorrectionDialog::Dialog_Data(
	TCorrectionDialog::TDialogData		* const dialog_data) const
{
	assert(dialog_data);
	
	dialog_data->temperature_data.reference_temperature_1 = d_reference_temperature_1_edit->text().toDouble();
	dialog_data->temperature_data.reference_temperature_2 = d_reference_temperature_2_edit->text().toDouble();
	dialog_data->temperature_data.reference_temperature_3 = d_reference_temperature_3_edit->text().toDouble();

	dialog_data->temperature_data.corrected_temperature_1 = d_meter_temperature_1_edit->text().toDouble();
	dialog_data->temperature_data.corrected_temperature_2 = d_meter_temperature_2_edit->text().toDouble();
	dialog_data->temperature_data.corrected_temperature_3 = d_meter_temperature_3_edit->text().toDouble();
	
	dialog_data->description = d_description_edit->text();
	
	dialog_data->calibration_due_date = d_calibration_due_edit->date();
}

void TCorrectionDialog::Reset(
	const TCorrectionDialog::TDialogData &dialog_data,
	const QStringList					&descriptions)
{
	d_description_edit->setText(dialog_data.description);
	
	d_calibration_due_edit->setDate(dialog_data.calibration_due_date);
	
	d_existing_descriptions = descriptions;
	
	d_reference_temperature_1_edit->setText(QString("%1").arg(dialog_data.temperature_data.reference_temperature_1,0,'f',2));
	d_reference_temperature_2_edit->setText(QString("%1").arg(dialog_data.temperature_data.reference_temperature_2,0,'f',2));
	d_reference_temperature_3_edit->setText(QString("%1").arg(dialog_data.temperature_data.reference_temperature_3,0,'f',2));

	d_meter_temperature_1_edit->setText(QString("%1").arg(dialog_data.temperature_data.corrected_temperature_1,0,'f',2));
	d_meter_temperature_2_edit->setText(QString("%1").arg(dialog_data.temperature_data.corrected_temperature_2,0,'f',2));
	d_meter_temperature_3_edit->setText(QString("%1").arg(dialog_data.temperature_data.corrected_temperature_3,0,'f',2));
	
	d_accept_button->setEnabled(false);
}

void TCorrectionDialog::Description_Changed(void)
{
	QString								text;
	
	text = d_description_edit->text();
	d_accept_button->setEnabled(false);
	
	if(text.length())
	{
		if(!d_existing_descriptions.contains(text,Qt::CaseInsensitive))
		{
			d_accept_button->setEnabled(true);
		}
	}
}

void TCorrectionDialog::Calibration_Date_Changed(void)
{
	d_accept_button->setEnabled(d_description_edit->text().length());
}

void TCorrectionDialog::Temperature_Changed(void)
{
	QLineEdit							*temp_edit;
	double								dval;
	bool								state;
	
	d_accept_button->setEnabled(false);

	temp_edit = qobject_cast<QLineEdit*>(this->sender());
	if(temp_edit)
	{
		dval = temp_edit->text().toDouble(&state);
		
		if(!state)
		{
			return;
		}
		
		if(dval < 0.0 || dval > 40.0)
		{
			return;
		}
		
		d_accept_button->setEnabled(d_description_edit->text().length());
	}
}



