/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////


#ifndef CORRECTIONDIALOGHEADERFILE
#define CORRECTIONDIALOGHEADERFILE

#include <QDialog>
#include <QDate>

#include "thermometerdata.h"

#include "correctiondata.h"

class QLineEdit;
class QPushButton;
class QDateEdit;

class TCorrectionDialog : public QDialog
{
Q_OBJECT

public:
	
struct TDialogData
{
	TThermometerData::TTemperatureData	temperature_data;
	QString								description;
	QDate								calibration_due_date;
};

// CREATORS
	TCorrectionDialog(const QWidget *parent=0, Qt::WindowFlags flags=Qt::WindowFlags());
	~TCorrectionDialog(void);

// ACCESSORS
	void Dialog_Data(TDialogData * const dialog_data) const;

// MANIPULATORS
	void Reset(const TDialogData &dialog_data,const QStringList &descriptions);

signals:

private slots:
	void Description_Changed(void);
	void Calibration_Date_Changed(void);
	void Temperature_Changed(void);

private:
	
	QDateEdit							*d_calibration_due_edit;
	QLineEdit							*d_description_edit;
	QLineEdit							*d_reference_temperature_1_edit;
	QLineEdit							*d_meter_temperature_1_edit;
	QLineEdit							*d_reference_temperature_2_edit;
	QLineEdit							*d_meter_temperature_2_edit;
	QLineEdit							*d_reference_temperature_3_edit;
	QLineEdit							*d_meter_temperature_3_edit;
	QPushButton							*d_accept_button;
	
	QStringList							d_existing_descriptions;
	
// NOT IMPLEMENTED
	TCorrectionDialog(const TCorrectionDialog&);
	TCorrectionDialog& operator=(const TCorrectionDialog&);
};

#endif
