/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////


#include <QApplication>
#include <QFrame>
#include <QGridLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QPushButton>
#include <QSpacerItem>

#include "inputdialog.h"

TInputDialog::TInputDialog(
	const QWidget						*parent,
	const Qt::WindowFlags				flags)
:QDialog(const_cast<QWidget*>(parent),flags)
{
	QFrame								*separator_hline;
	QGridLayout							*dialog_layout;
	QHBoxLayout							*button_hlayout;
	QPushButton							*cancel_button;
	QSpacerItem							*button_hspacer;
	QSpacerItem							*dialog_vspacer;


	this->resize(384,113);

	dialog_layout = new QGridLayout(this);

	d_input_edit = new QLineEdit(this);
	d_input_edit->setAlignment(Qt::AlignCenter);
	dialog_layout->addWidget(d_input_edit,0,1,1,1);

	d_input_description_label = new QLabel(this);
	dialog_layout->addWidget(d_input_description_label,0,0,1,1);

	d_input_units_label = new QLabel(this);
	dialog_layout->addWidget(d_input_units_label,0,2,1,1);

	separator_hline = new QFrame(this);
	separator_hline->setFrameShape(QFrame::HLine);
	separator_hline->setFrameShadow(QFrame::Sunken);
	dialog_layout->addWidget(separator_hline,1,0,1,3);

	dialog_vspacer = new QSpacerItem(0,0,QSizePolicy::Minimum,QSizePolicy::Expanding);
	dialog_layout->addItem(dialog_vspacer,2,0,1,3);

	button_hlayout = new QHBoxLayout();

	button_hspacer = new QSpacerItem(0,0,QSizePolicy::Expanding,QSizePolicy::Minimum);
	button_hlayout->addItem(button_hspacer);

	cancel_button = new QPushButton(this);
	cancel_button->setAutoDefault(false);
	button_hlayout->addWidget(cancel_button);

	d_accept_button = new QPushButton(this);
	d_accept_button->setAutoDefault(true);
	d_accept_button->setDefault(true);
	button_hlayout->addWidget(d_accept_button);
	dialog_layout->addLayout(button_hlayout,4,0,1,3);

	this->setWindowTitle(QStringLiteral("Input Dialog"));
	d_input_description_label->setText(QStringLiteral("Calibration Uncertainty (k=2):"));
	d_input_units_label->setText(QStringLiteral("C"));
	cancel_button->setText(QStringLiteral("Cancel"));
	d_accept_button->setText(QStringLiteral("Accept"));
	
	connect(cancel_button,&QPushButton::clicked,this,&TInputDialog::reject);
	connect(d_accept_button,&QPushButton::clicked,this,&TInputDialog::accept);

	connect(d_input_edit,&QLineEdit::textEdited,this,&TInputDialog::Value_Changed);
}

TInputDialog::~TInputDialog(void)
{
}

double TInputDialog::Uc_Value(void) const
{
	return d_input_edit->text().toDouble();
}

void TInputDialog::Reset(
	const double 						&uc_value)
{
	d_input_edit->setText(QString("%1").arg(uc_value,0,'f',2));
}

void TInputDialog::Value_Changed(
	const QString						&text)
{
	double								dval;
	bool								state;
	static const double					MAX_UC(1.0);
	
	dval = text.toDouble(&state);
	
	if(state && dval < MAX_UC && !(dval < 0.0))
	{
		d_accept_button->setEnabled(true);
	}
	else
	{
		d_accept_button->setEnabled(false);
	}
}
