/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////


#ifndef TEMPERATURERECORDHEADERFILE
#define TEMPERATURERECORDHEADERFILE

#include <QMainWindow>

#include "thermometerdata.h"
#include "correctiondata.h"

class QAction;
class QCheckBox;
class QComboBox;
class QLineEdit;
class QPlainTextEdit;
class QToolButton;
class QPushButton;
class QSettings;
class QCloseEvent;

class TCorrectionDialog;
class TMessageBox;
class TInputDialog;

class TTemperatureRecord : public QMainWindow
{
Q_OBJECT

public:

// CREATORS
	TTemperatureRecord(const QWidget *parent=0, Qt::WindowFlags flags=Qt::WindowFlags());
	virtual ~TTemperatureRecord(void);

// ACCESSORS

// MANIPULATORS

signals:

private slots:
	void Browse_Path(void);
	void Edit_Thermometer(void);
	void Remove_Thermometer(void);
	void Thermometer_Selection_Changed(int);
	void Temperature_Input_Changed(void);
	void Write_Temperature_Entry(void);
	void Evaluate_Corrections(void);
	void Evaluate_Uncertainty(void);

protected:
	virtual void closeEvent(QCloseEvent *event);

private:
	QAction								*d_action_edit_thermometer;
	QAction								*d_action_remove_thermometer;
	QAction								*d_action_file_quit;
	QCheckBox							*d_temperature_part_check;
	QCheckBox							*d_temperature_x_check;
	QCheckBox							*d_temperature_y_check;
	QCheckBox							*d_temperature_z_check;
	QComboBox							*d_thermometer_combo;
	QLineEdit							*d_corrected_part_edit;
	QLineEdit							*d_description_edit;
	QLineEdit							*d_corrected_x_edit;
	QLineEdit							*d_corrected_y_edit;
	QLineEdit							*d_corrected_z_edit;
	QLineEdit							*d_meter_part_edit;
	QLineEdit							*d_meter_x_edit;
	QLineEdit							*d_meter_y_edit;
	QLineEdit							*d_meter_z_edit;
	QLineEdit							*d_record_file_edit;
	QLineEdit							*d_record_path_edit;
	QPlainTextEdit						*d_log_edit;
	QToolButton							*d_path_browse_button;
	QToolButton							*d_thermometer_edit_button;
	QToolButton							*d_thermometer_delete_button;
	QPushButton							*d_write_button;
	QPushButton							*d_evaluate_corrections_button;
	QPushButton							*d_evaluate_uncertainty_button;
	QSettings							*d_settings;
	
	TMessageBox							*d_msg_box;
	TCorrectionDialog					*d_correction_dialog;
	TInputDialog						*d_input_dialog;
	
	TThermometerData					d_thermometer_data;
	bool								d_active_thermometer_usable;
	TCorrectionData						d_active_correction_data;

	QString								d_application_path;
	QString								d_data_path;
	QString								d_current_path;
	
	void Update_All_Temperature_Corrections(void);
	
	void Clear_Log(void);
	void Add_Log_Text(const QString &text);


// NOT IMPLEMENTED
	TTemperatureRecord(const TTemperatureRecord&);
	TTemperatureRecord& operator=(const TTemperatureRecord&);
};

#endif
