/////////////////////////////////////////////////////////////////////
//
//            X   X           X
//           XXX XX         XX
//          XXXXXXXX      XXX
//         XX X XXXXXXXXXXX
//        XXXXX XXXXXXXXX
//       XXXXX XXXXXXXXXX
//            XXX XXX XXX
//           XXX XX   XX
//           X   X     X
//
//    Copyright (C) 2003-2023  Ron Jakl
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////

#ifndef THERMOMETERDATAHEADERFILE
#define THERMOMETERDATAHEADERFILE

#include <QString>
#include <QStringList>
#include <QDate>
#include <vector>

#include "../../core/vector2.h"

#include "correctiondata.h"

class TXmlFile;

class TThermometerData
{
public:
	
struct TTemperatureData
{
	double								reference_temperature_1;
	double								reference_temperature_2;
	double								reference_temperature_3;
	
	double								corrected_temperature_1;
	double								corrected_temperature_2;
	double								corrected_temperature_3;
};
	
// CREATORS
	TThermometerData(void);
	~TThermometerData(void);
	
// ACCESSORS
	QString Last_Error(void) const {return d_last_error;}
	int Checksum_Error_Count(void) const {return d_checksum_error_count;}
	
	QStringList Description_List(void) const;
	
	bool Get_Temperature_Data(const QString &description,TTemperatureData * const temperature_data) const;	// returns default data if description not found
	
	TCorrectionData Get_Correction_Data(const QString &description) const;
	QDate Get_Calibration_Due_Date(const QString &description) const;
	bool Is_Calibration_Current(const QString &description) const;
	
	bool Save_Data(const QString &file_name) const;
	
// MANIPULATORS
	bool Load_Data(const QString &file_name);
	
	void Set_Temperature_Data(const QString &description,const TTemperatureData &temperature_data);	// adds new or replace existing
	void Set_Calibration_Due_Date(const QString &description,const QDate &date);
	
	void Remove_Item(const QString &description);
	
private:
	
struct TThermometerDataItem
{
	QString								description;
	TCorrectionData						correction_data;
	QDate								calibration_due_date;
};
	
	std::vector<TThermometerDataItem>	d_thermometer_data_items;
	
	int									d_checksum_error_count;
	mutable QString						d_last_error;
	
	QString Checksum(const TThermometerDataItem &thermometer_data_item) const;

	void Load_Data_V1(TXmlFile * const xml_file);
	
// NOT IMPLEMENTED
	TThermometerData(const TThermometerData&);
	TThermometerData& operator=(const TThermometerData&);
};

#endif
